#!/usr/bin/env python3
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#
# from datetime import date


"""OpenDataLab python SDK version info."""

__url__ = "https://opendatalab.org.cn"
__version__ = "0.0.4"
__svc__ = '2.0'
odl_clientId = "kmz3bkwzlaa3wrq8pvwa"
uaa_url_prefix = "https://sso.openxlab.org.cn/gw/uaa-be"